@@ -47,7 +47,7 @@ PamAuthenticators::PamAuthenticators(std::unique_ptr<PamAuthenticator> &&interac
     });
     connect(d->interactive.get(), &PamAuthenticator::failed, this, [this] {
         qCDebug(KSCREENLOCKER_GREET) << "PamAuthenticators: Failure from interactive authenticator" << qUtf8Printable(d->interactive->service());
-        setState(AuthenticatorsState::Failed);
+        setState(AuthenticatorsState::Idle);
         d->cancelNoninteractive();
         Q_EMIT failed(PamAuthenticator::NoninteractiveAuthenticatorType::None, d->interactive.get());
     });
@@ -64,8 +64,6 @@ PamAuthenticators::PamAuthenticators(std::unique_ptr<PamAuthenticator> &&interac
         });
         connect(noninteractive.get(), &PamAuthenticator::failed, this, [this, &noninteractive] {
             qCDebug(KSCREENLOCKER_GREET) << "PamAuthenticators: Non-interactive authenticator" << qUtf8Printable(noninteractive->service()) << "failed";
-            setState(AuthenticatorsState::Failed);
-            d->cancelNoninteractive();
             Q_EMIT failed(noninteractive->authenticatorType(), noninteractive.get());
         });
         connect(noninteractive.get(), &PamAuthenticator::infoMessage, this, [this, &noninteractive]() {
